@@ -7,6 +7,11 @@ from containers import TokenDetails, Single, Group
 # Regex for matching whitespace
 regexes = {'whitespace': re.compile('\s+')}
 
+class WildCard(object):
+    """Used to determine if tokens should be inserted untill ignored token"""
+    def __repr__(self):
+        return "<WildCard>"
+    
 class Tracker(object):
     """Keep track of what each next token should mean"""
     def __init__(self, result, tokens):
@@ -28,6 +33,7 @@ class Tracker(object):
         self.adjust_indent_at = []
         
         self.indent_type = ' '
+        self.insert_till = None
         self.after_space = True
         self.inserted_line = False
         self.just_ended_container = False
@@ -67,6 +73,10 @@ class Tracker(object):
             # Change indentation as necessary
             self.determine_indentation()
             
+            # See if we are force inserting this token
+            if self.forced_insert():
+                return
+            
             # If we have a newline after an inserted line, then we don't need to worry about semicolons
             if self.inserted_line and self.current.tokenum == NEWLINE:
                 self.inserted_line = False
@@ -197,16 +207,55 @@ class Tracker(object):
     
     def ignore_token(self):
         """Determine if we should ignore current token"""
-        if self.ignore_next:
+        def get_next_ignore(remove=False):
+            """Get next ignore from ignore_next and remove from ignore_next"""
             next_ignore = self.ignore_next
-            if type(next_ignore) in (list, tuple):
-                next_ignore = self.ignore_next.pop(0)
             
-            if next_ignore == (self.current.tokenum, self.current.value):
-                return True
+            # Just want to return it, don't want to remove yet
+            if not remove:
+                if type(self.ignore_next) in (list, tuple):
+                    next_ignore = self.ignore_next[0]
+                return next_ignore
+            
+            # Want to remove it from ignore_next
+            if type(next_ignore) in (list, tuple) and next_ignore:
+                next_ignore = self.ignore_next.pop(0)
+            elif not next_ignore:
+                self.next_ignore = None
+                next_ignore = None
             else:
                 self.next_ignore = None
-                return False
+            
+            return next_ignore
+        
+        # If we have tokens to be ignored and we're not just inserting till some token
+        if not self.insert_till and self.ignore_next:
+            # Determine what the next ignore is
+            next_ignore = get_next_ignore()
+            if next_ignore == (self.current.tokenum, self.current.value):
+                # Found the next ignore token, remove it from the stack
+                # So that the next ignorable token can be considered
+                get_next_ignore(remove=True)
+                return True
+            else:
+                # If not a wildcard, then return now
+                if type(next_ignore) is not WildCard:
+                    return False
+                
+                # Go through tokens untill we find one that isn't a wildcard
+                while type(next_ignore) == WildCard:
+                    next_ignore = get_next_ignore(remove=True)
+                
+                # If the next token is next ignore then we're done here!
+                if next_ignore == (self.current.tokenum, self.current.value):
+                    get_next_ignore(remove=True)
+                    return True
+                else:
+                    # If there is another token to ignore, then consider the wildcard
+                    # And keep inserting till we reach this next ignorable token
+                    if next_ignore:
+                        self.insert_till = next_ignore
+                    return False
     
     def make_method_names(self):
         """Create tokens for setting __testname__ on functions"""
@@ -230,6 +279,34 @@ class Tracker(object):
                     lst.extend(self.tokens.make_describe_attr(group.kls_name))
         
         return lst
+            
+    def forced_insert(self):
+        """
+            Insert tokens if self.insert_till hasn't been reached yet
+            Will respect self.inserted_line and make sure token is inserted before it
+            Returns True if it appends anything or if it reached the insert_till token
+        """
+        # If we have any tokens we are waiting for
+        if self.insert_till:
+            # Determine where to append this token
+            append_at = -1
+            if self.inserted_line:
+                append_at = -self.inserted_line+1
+            
+            # Reset insert_till if we found it
+            if self.current.tokenum == self.insert_till[0] and self.current.value == self.insert_till[1]:
+                self.insert_till = None
+            else:
+                # Adjust self.adjust_indent_at to take into account the new token
+                for index, value in enumerate(self.adjust_indent_at):
+                    if value < len(self.result) - append_at:
+                        self.adjust_indent_at[index] = value + 1
+                
+                # Insert the new token
+                self.result.insert(append_at, (self.current.tokenum, self.current.value))
+            
+            # We appended the token
+            return True
     
     ########################
     ###   ADD TOKENS
@@ -264,17 +341,15 @@ class Tracker(object):
             self.adjust_indent_at.append(len(self.result) + 2)
             
             # Add tokens for super call
-            self.result.extend(self.tokens.make_super(self.indent_type * self.current.scol, self.groups.kls_name, value))
+            tokens_for_super = self.tokens.make_super(self.indent_type * self.current.scol, self.groups.kls_name, value)
+            self.result.extend(tokens_for_super)
             
             # Tell the machine we inserted a line
-            self.inserted_line = True
+            self.inserted_line = len(tokens_for_super)
             
             # Make sure colon and newline are ignored
             # Already added as part of making super
-            self.ignore_next = [
-                  (OP, ':')
-                , (NEWLINE, '\n')
-                ]
+            self.ignore_next = [(OP, ':'), WildCard(), (NEWLINE, '\n')]
     
     def add_tokens_for_group(self, with_pass=False):
         """Add the tokens for the group signature"""
